/*
 * serLCD.c
 *
 *  Created on: Mar 29, 2021
 *      Author: ankitjayant
 */


#include "SerLCD.h"
#include "main.h"




uint8_t _i2cAddr = DISPLAY_ADDRESS1;
I2C_HandleTypeDef *_i2cHandler;
uint8_t _displayControl = LCD_DISPLAYON | LCD_CURSOROFF | LCD_BLINKOFF;
uint8_t _displayMode = LCD_ENTRYLEFT | LCD_ENTRYSHIFTDECREMENT;

/*
 *******************************************************************************
 * initial display functions
 *******************************************************************************
 */


/**
 *  THIS IS THE MOST IMPORTANT THING, INITIALIZATION HOW TO
 *  PASS THE HANDLER AND SET UP A DATA STREAM, MODEL THE AUDIO CODEC AFTER THIS STUFF
 */
uint8_t displayInit(I2C_HandleTypeDef *hi2c)
{
	uint8_t retval = LCD_OK;

	_i2cHandler = hi2c;			// pointer for i2c handler is stored in library

	// create i2c data stream
    uint8_t TransmitData[6] = {SPECIAL_COMMAND,									// special command character
    						   LCD_DISPLAYCONTROL | _displayControl, 			// display command
							   SPECIAL_COMMAND, 								// command character
							   LCD_ENTRYMODESET | _displayMode, 				// entry mode command
							   SETTING_COMMAND, 								// Put LCD into setting mode
							   CLEAR_COMMAND 									// clear display
    						   };

    // transmission of data stream
    if(HAL_I2C_Master_Transmit(_i2cHandler, DISPLAY_ADDRESS1<<1, TransmitData, sizeof(TransmitData), 100) != HAL_OK)		// transmit data
    	retval = LCD_ERROR;

    HAL_Delay(50);

    return retval;
}



/*
 *******************************************************************************
 * move on the display
 *******************************************************************************
 */


/**
 * go back to start of screen and start writing,
 */
uint8_t displayHome()
{
	return displayCommand(LCD_RETURNHOME);
}



/**
 * row/col set cursor on screen
 */
uint8_t displaySetCursor(uint8_t col, uint8_t row)
{
  uint8_t row_offsets[] = {0x00, 0x40, 0x14, 0x54};

  return displaySpecialCommand(LCD_SETDDRAMADDR | (col + row_offsets[row]));
}






/**
 * scroll left/scrol right
 */
uint8_t displayScrollDisplayLeft()
{
  return displaySpecialCommand(LCD_CURSORSHIFT | LCD_DISPLAYMOVE | LCD_MOVELEFT);
}



/**
 * scroll left/right
 */
uint8_t displayScrollDisplayRight()
{
  return displaySpecialCommand(LCD_CURSORSHIFT | LCD_DISPLAYMOVE | LCD_MOVERIGHT);
}



/**
 *  make autscroling work
 */
uint8_t displayAutoscroll()
{
	 _displayMode |= LCD_ENTRYSHIFTINCREMENT;
	 return displaySpecialCommand(LCD_ENTRYMODESET | _displayMode);
}



/**
 * make screen stagnant
 */
uint8_t displayNoAutoscroll()
{
	 _displayMode &= ~LCD_ENTRYSHIFTINCREMENT;
	 return displaySpecialCommand(LCD_ENTRYMODESET | _displayMode);
}



/*
 *******************************************************************************
 * write on the display
 *******************************************************************************
 */


/**
 * clear the display, again very useful and nice
 */
uint8_t displayClear()
{
	uint8_t retval = displayCommand(CLEAR_COMMAND);
	HAL_Delay(10);
	return retval;
}



/**
 *  write a byte of data, i used this while testing, pretty good
 */
uint8_t displayWrite(uint8_t b)
{
	uint8_t retval = LCD_OK;

	// transmission of data stream
	if(HAL_I2C_Master_Transmit(_i2cHandler, DISPLAY_ADDRESS1<<1, b, 1, 100) != HAL_OK)
	  retval = LCD_ERROR;
	HAL_Delay(10); // wait a bit

    return retval;
}



/**
 * wite a string as char buffer to the screen, this is the main fuction
 *
 */
uint8_t displayWriteString(char *buffer)
{
	uint8_t retval = LCD_OK;

	// transmission of data stream
	if(HAL_I2C_Master_Transmit(_i2cHandler, DISPLAY_ADDRESS1<<1, buffer, strlen(buffer), 100) != HAL_OK)		// transmit data
	  retval = LCD_ERROR;
	HAL_Delay(10); //This takes a bit longer

    return retval;
}





/**
 *  write a single character
 */
uint8_t displayWriteChar(uint8_t location)
{
  location &= 0x7; // we only have 8 locations 0-7

  return displayCommand(35 + location);
}





/*
 * Change the I2C Address. 0x72 is the default, use a different one AVI !! :)
 */
uint8_t displaySetAddress(uint8_t new_addr)
{
	uint8_t retval = LCD_OK;

	// create i2c data stream
    uint8_t TransmitData[3] = {SETTING_COMMAND,									// special command character
    						   ADDRESS_COMMAND, 								// send address command
							   new_addr											// set new address
							   };

    // transmission of data stream
    if(HAL_I2C_Master_Transmit(_i2cHandler, DISPLAY_ADDRESS1<<1, TransmitData, sizeof(TransmitData), 100) != HAL_OK)		// transmit data
    	retval = LCD_ERROR;


    if(retval == LCD_OK)
    	_i2cAddr = new_addr;

    HAL_Delay(50);	 //This may take awhile

	return retval;
}



/*
 *******************************************************************************
 * edit splash screen
 *******************************************************************************
 */





/**
 *  save homescreen
 */
uint8_t displaySaveSplash()
{
	uint8_t retval = LCD_OK;

	// create i2c data stream
    uint8_t TransmitData[2] = {SETTING_COMMAND,									// special command character
    						   SAVE_CURRENT_DISPLAY_AS_SPLASH, 					// set Ctrl+j character
							   };

    // transmission of data stream
    if(HAL_I2C_Master_Transmit(_i2cHandler, DISPLAY_ADDRESS1<<1, TransmitData, sizeof(TransmitData), 100) != HAL_OK)		// transmit data
    	retval = LCD_ERROR;
    HAL_Delay(10);

	return retval;
}



/*
 *******************************************************************************
 * only for internal use - don't touch!
 * got these from online sources, set up the base for alot of other commands as you can see
 *******************************************************************************
 */


/**
 * @brief	Send a command to the display.
 * 			Used by other functions.
 * @retval	status 	- 0...OK, other...error
 */
uint8_t displayCommand(uint8_t command)
{
	uint8_t retval = LCD_OK;
	uint8_t TransmitData[2] = {SETTING_COMMAND, command}; 												// create data stream
	if(HAL_I2C_Master_Transmit(_i2cHandler, DISPLAY_ADDRESS1<<1, TransmitData, sizeof(TransmitData), 100) != HAL_OK)		// transmit data
		retval = LCD_ERROR;
	HAL_Delay(10); //Wait a bit longer for special display commands
	return retval;
}



/**
 * @brief	Send a special command to the display.  Used by other functions.
 * @retval	status (0...OK, other...error)
 */
uint8_t displaySpecialCommand(uint8_t command)
{
	uint8_t retval = LCD_OK;
	uint8_t TransmitData[2] = {SPECIAL_COMMAND, command}; 												// create data stream
	if(HAL_I2C_Master_Transmit(_i2cHandler, DISPLAY_ADDRESS1<<1, TransmitData, sizeof(TransmitData), 100) != HAL_OK)		// transmit data
		retval = LCD_ERROR;
	HAL_Delay(50); //Wait a bit longer for special display commands
	return retval;
}
